@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-favicon-cache.c,v 1.55 2005/10/16 18:52:26 chpe Exp $
+ *  $Id: ephy-favicon-cache.c,v 1.58 2006/06/01 19:35:31 jframeau Exp $
  */
 
 #include "config.h"
@@ -196,7 +196,11 @@ icon_is_obsolete (EphyNode *node, GDate *now)
 		(node, EPHY_NODE_FAVICON_PROP_LAST_USED);
 
 	g_date_clear (&date, 1);
+#if GLIB_CHECK_VERSION (2,9,0)
+        g_date_set_time_t (&date, last_visit);
+#else
 	g_date_set_time (&date, last_visit);
+#endif
 
 	return (g_date_days_between (&date, now) >=
 		EPHY_FAVICON_CACHE_OBSOLETE_DAYS);
@@ -236,7 +240,11 @@ remove_obsolete_icons (EphyFaviconCache *cache,
 	GDate current_date;
 
 	g_date_clear (&current_date, 1);
+#if GLIB_CHECK_VERSION (2,9,0)
+        g_date_set_time_t (&current_date, time (NULL));
+#else
 	g_date_set_time (&current_date, time (NULL));
+#endif
 
 	children = ephy_node_get_children (priv->icons);
 	for (i = (int) children->len - 1; i >= 0; i--)
@@ -545,7 +553,9 @@ ephy_favicon_cache_download (EphyFaviconCache *cache,
 	ephy_embed_persist_set_dest (persist, dest);
 	ephy_embed_persist_set_flags (persist, EPHY_EMBED_PERSIST_NO_VIEW |
 					       EPHY_EMBED_PERSIST_NO_CERTDIALOGS |
-					       EPHY_EMBED_PERSIST_DO_CONVERSION);
+					       EPHY_EMBED_PERSIST_DO_CONVERSION |
+					       EPHY_EMBED_PERSIST_NO_COOKIES
+				               );
 	ephy_embed_persist_set_max_size (persist, EPHY_FAVICON_MAX_SIZE);
 	ephy_embed_persist_set_source (persist, favicon_url);
 
@@ -588,12 +598,8 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 
 	if (url == NULL) return NULL;
 
-#ifdef HAVE_GECKO_1_8
 	if (!g_str_has_prefix (url, "http://") &&
 	    !g_str_has_prefix (url, "https://")) return NULL;
-#else
-	if (!g_str_has_prefix (url, "http://")) return NULL;
-#endif
 
 	priv->requests += 1;
 
