@@ -17,26 +17,27 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-notifiers.h,v 1.8 2005/07/04 12:22:40 chpe Exp $
+ *  $Id: mozilla-notifiers.h,v 1.10 2006/05/10 18:07:00 chpe Exp $
  */
 
 #ifndef MOZILLA_NOTIFIERS_H
 #define MOZILLA_NOTIFIERS_H
 
+#include <glib.h>
 #include <glib-object.h>
 #include <gconf/gconf.h>
 
 G_BEGIN_DECLS
 
-typedef gboolean (* PrefValueTransformFunc)	(GConfValue *, GValue *, gpointer);
+typedef gboolean (* PrefValueTransformFunc)	(GConfEntry*, GValue*, gpointer);
 
-gboolean mozilla_notifier_transform_bool	(GConfValue *, GValue *, gpointer);
+gboolean mozilla_notifier_transform_bool	(GConfEntry*, GValue*, gpointer);
 
-gboolean mozilla_notifier_transform_bool_invert	(GConfValue *, GValue *, gpointer);
+gboolean mozilla_notifier_transform_bool_invert	(GConfEntry*, GValue*, gpointer);
 
-gboolean mozilla_notifier_transform_int		(GConfValue *, GValue *, gpointer);
+gboolean mozilla_notifier_transform_int		(GConfEntry*, GValue*, gpointer);
 
-gboolean mozilla_notifier_transform_string	(GConfValue *, GValue *, gpointer);
+gboolean mozilla_notifier_transform_string	(GConfEntry*, GValue*, gpointer);
 
 guint	 mozilla_notifier_add		(const char *gconf_key,
 					 const char *mozilla_pref,
@@ -48,7 +49,7 @@ void	 mozilla_notifier_remove	(guint id);
 gboolean mozilla_pref_set		(const char *pref,
 					 const GValue *value);
 
-void	 mozilla_notifiers_init		(void);
+gboolean mozilla_notifiers_init		(void);
 
 void	 mozilla_notifiers_shutdown	(void);
 
