@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.173 2006/03/17 14:08:01 wbolster Exp $
+ *  $Id: ephy-tab.c,v 1.175 2006/08/07 20:35:37 chpe Exp $
  */
 
 #include "config.h"
@@ -78,6 +78,8 @@
 
 struct _EphyTabPrivate
 {
+	guint id;
+
 	char *status_message;
 	char *link_message;
 	char *address;
@@ -146,7 +148,15 @@ typedef struct
 	char *features;
 } PopupInfo;
 
-static GObjectClass *parent_class = NULL;
+static GObjectClass *parent_class;
+
+/* We need to assign unique IDs to tabs, otherwise accels get confused in the
+ * tabs menu (bug #339548). We could use a serial #, but the ID is used in the
+ * action name which is stored in a GQuark and so we should use them sparingly.
+ */
+
+static GArray *tabs_id_array = NULL;
+static guint n_tabs = 0;
 
 /* internal functions, accessible only from this file */
 static void	ephy_tab_set_load_status	(EphyTab *tab,
@@ -821,6 +831,7 @@ ephy_tab_finalize (GObject *object)
 {
 	EphyTab *tab = EPHY_TAB (object);
 	EphyTabPrivate *priv = tab->priv;
+	guint id = priv->id;
 
 	if (priv->icon != NULL)
 	{
@@ -844,6 +855,14 @@ ephy_tab_finalize (GObject *object)
 #endif
 
 	LOG ("EphyTab finalized %p", tab);
+
+	/* Remove the ID */
+	g_array_index (tabs_id_array, gpointer, id) = NULL;
+	if (--n_tabs == 0)
+	{
+		g_array_free (tabs_id_array, TRUE);
+		tabs_id_array = NULL;
+	}
 }
 
 static gboolean
@@ -1886,7 +1905,7 @@ open_link_in_new (EphyTab *tab,
 	return FALSE;
 }
 
-static void
+static gboolean
 save_property_url (EphyEmbed *embed,
 		   EphyEmbedEvent *event,
 		   const char *property,
@@ -1899,6 +1918,8 @@ save_property_url (EphyEmbed *embed,
 	value = ephy_embed_event_get_property (event, property);
 	location = g_value_get_string (value);
 
+	if (!address_has_web_scheme (location)) return FALSE;
+
 	persist = EPHY_EMBED_PERSIST
 		(ephy_embed_factory_new_object (EPHY_TYPE_EMBED_PERSIST));
 
@@ -1910,6 +1931,8 @@ save_property_url (EphyEmbed *embed,
 	ephy_embed_persist_save (persist);
 
 	g_object_unref (G_OBJECT(persist));
+
+	return TRUE;
 }
 
 static void
@@ -1988,14 +2011,14 @@ ephy_tab_dom_mouse_click_cb (EphyEmbed *embed,
 	/* shift+click saves the link target */
 	else if (is_link && is_left_click && with_shift)
 	{
-		save_property_url (embed, event, "link", CONF_STATE_SAVE_DIR);
+		handled = save_property_url (embed, event, "link", CONF_STATE_SAVE_DIR);
 	}
 	/* shift+click saves the non-link image
 	 * Note: pressing enter to submit a form synthesizes a mouse click event
 	 */
 	else if (is_image && is_left_click && with_shift && !is_input)
 	{
-		save_property_url (embed, event, "image", CONF_STATE_SAVE_IMAGE_DIR);
+		handled = save_property_url (embed, event, "image", CONF_STATE_SAVE_IMAGE_DIR);
 	}
 	/* middle click opens the selection url */
 	else if (is_middle_clickable && is_middle_click && middle_click_opens)
@@ -2039,11 +2062,41 @@ ephy_tab_init (EphyTab *tab)
 	EphyTabPrivate *priv;
 	GObject *embed;
 	EphyFaviconCache *cache;
+	guint id;
 
 	LOG ("EphyTab initialising %p", tab);
 
 	priv = tab->priv = EPHY_TAB_GET_PRIVATE (tab);
 
+	/* Make tab ID */
+	++n_tabs;
+
+	if (tabs_id_array == NULL)
+	{
+		tabs_id_array = g_array_sized_new (FALSE /* zero-terminate */,
+					       TRUE /* clear */,
+					       sizeof (gpointer),
+					       64 /* initial size */);
+	}
+
+	for (id = 0; id < tabs_id_array->len; ++id)
+	{
+		if (g_array_index (tabs_id_array, gpointer, id) == NULL) break;
+	}
+
+	priv->id = id;
+
+	/* Grow array if necessary */
+	if (id >= tabs_id_array->len)
+	{
+		g_array_append_val (tabs_id_array, tab);
+		g_assert (g_array_index (tabs_id_array, gpointer, id) == tab);
+	}
+	else
+	{
+		g_array_index (tabs_id_array, gpointer, id) = tab;
+	}
+
 	tab->priv->total_requests = 0;
 	tab->priv->cur_requests = 0;
 	tab->priv->width = -1;
@@ -2466,3 +2519,12 @@ ephy_tab_get_zoom (EphyTab *tab)
 
 	return tab->priv->zoom;
 }
+
+/* private */
+guint
+_ephy_tab_get_id (EphyTab *tab)
+{
+	g_return_val_if_fail (EPHY_IS_TAB (tab), (guint) -1);
+
+	return tab->priv->id;
+}
