@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: pdm-dialog.c,v 1.62 2005/12/08 21:32:52 chpe Exp $
+ *  $Id: pdm-dialog.c,v 1.63 2006/05/30 21:12:11 fpeters Exp $
  */
 
 #include "config.h"
@@ -594,6 +594,7 @@ pdm_dialog_cookies_construct (PdmActionInfo *info)
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_reorderable (column, TRUE);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_sort_column_id (column, COL_COOKIES_HOST_KEY);
 
 	gtk_tree_view_insert_column_with_attributes (treeview,
 						     TV_COL_COOKIES_NAME,
@@ -605,6 +606,7 @@ pdm_dialog_cookies_construct (PdmActionInfo *info)
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_reorderable (column, TRUE);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_sort_column_id (column, COL_COOKIES_NAME);
 
 	gtk_tree_view_set_enable_search (treeview, TRUE);
 	gtk_tree_view_set_search_column (treeview, COL_COOKIES_HOST);
